/*

C Source: fillcd

File:     fillcd.c

Description:

Use the Astrometric Information in a GASP/Digital Sky Survey
frame header to create a suitable CD matrix and CRPIX and CRVAL
values.

This module is intended to be integrated with the "getimage"
program and assumes the same data structures.

It is based on the STSDAS makewcs task.

History:

Test version, 16th February 1995, Richard Hook, ST-ECF
	      17-May-95 removed f2c dependencies, Miguel Albrecht
*/

#include "header.h"  /* GASP header definitions */
#include <math.h>    /* Standard math library header */

#define abs(x) ((x) >= 0 ? (x) : -(x))
#define dabs(x) (double)abs(x)
#define sign(x) ((x) < 0 ? -1. : 1.)
#ifndef min
#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#endif
#define dmin(a,b) (double)min(a,b)
#define dmax(a,b) (double)max(a,b)

#define RD2DEG  180./3.141592653589793

extern void fillcd(h)
HEADER h;
{
    double crpix1,crpix2,crval1,crval2,cdelt1,cdelt2,crota2;
    double im_x_center_pix,im_y_center_pix, a, b;
    double object_mag=0.0;
    double object_col=0.0;
    double cdmatx[4],pcmatx[4];
    int no_crval1, no_crval2;
    int no_crpix1, no_crpix2;
    int no_cd1_1, no_cd2_2, no_cd1_2, no_cd2_1;
    int no_ctype1, no_ctype2;
    int no_pcmat, no_cdelt1, no_cdelt2;
    int i,j;
    char keyword[9],hlast[81];
    
    /* First calculate suitable CRPIX values and then the centre
       pixel values in the sub-array */
    crpix1 = h->section_x_length/2.0;
    crpix2 = h->section_y_length/2.0;
    
    im_x_center_pix = h->section_x_corner+crpix1-0.5;
    im_y_center_pix = h->section_y_corner+crpix2-0.5;
    
    /* Now use the astrometric coefficients which we assume are in
       the header structure already to calculate the corresponding
       equatorial position of the reference pixel */
    ccgseq_(&h->plt_center_ra,
	    &h->plt_center_dec,
	    &h->ppo_coeff[2],
	    &h->ppo_coeff[5],
	    &h->x_pixel_size,
	    &h->y_pixel_size,
	    &h->plt_scale,
	    h->amd_x_coeff,
	    h->amd_y_coeff,
	    &im_x_center_pix,&im_y_center_pix,
	    &object_mag,&object_col,
	    &crval1,&crval2);
    
    /* We now have the information we need to calculate the CD
       matrix as well */
    calcds_(&h->plt_center_ra,
	    &h->plt_center_dec,
	    &h->ppo_coeff[2],
	    &h->ppo_coeff[5],
	    &h->section_x_corner,
	    &h->section_y_corner,
	    &h->x_pixel_size,
	    &h->y_pixel_size,
	    &h->plt_scale,
	    &h->section_x_length,
	    &h->section_y_length,
	    &crval1,&crval2,
	    h->amd_x_coeff,
	    h->amd_y_coeff,
	    cdmatx);
    
    /* Convert to degrees from radians and calculate CDELTi */
    
    crval1 *= RD2DEG;
    crval2 *= RD2DEG;
    
    cdelt1 = - sqrt(cdmatx[0]*cdmatx[0] + cdmatx[2]*cdmatx[2]);
    cdelt2 = sqrt(cdmatx[1]*cdmatx[1] + cdmatx[3]*cdmatx[3]);
    
    /* Now compute the PC matrix from the CD matrix ... */
    
    pcmatx[0] = cdmatx[0] / cdelt1;
    pcmatx[1] = cdmatx[1] / cdelt1;
    pcmatx[2] = cdmatx[2] / cdelt2;
    pcmatx[3] = cdmatx[3] / cdelt2;
    
    /* ... and also CROTA2
       following the eqs 4.2-4.5 (not 4.8-4.9) given in 
       NOST FITS User's Guide, Version 3.1, May 1994, pages 49-50
       BEWARE: we use atan instead of atan2 because of quadrant error.
    */

    a = atan(sign(cdelt1*cdelt2)*cdmatx[1]/cdmatx[3]) * RD2DEG;
    b = atan(-sign(cdelt1*cdelt2)*cdmatx[2]/cdmatx[0]) * RD2DEG;

    crota2 = (a+b)/2.;

    /*
     * insert the new keywords just before
     *  the last line of the header
     */
    i = h->nhlines-1;
    for (j=0; j<sizeof(hlast); j++) hlast[j] = h->hlines[i][j];

    /* CTYPE1 */
    sprintf(h->hlines[i], "%-8.8s= %-20s /%-50.50s",
            "CTYPE1", "'RA---TAN'", "R.A. in tangent plane projection");
    i = i + 1;

    /* CTYPE2 */
    sprintf(h->hlines[i], "%-8.8s= %-20s /%-50.50s",
            "CTYPE2", "'DEC--TAN'", "DEC. in tangent plane projection");
    i = i + 1;

    /* CRPIX1 */
    sprintf(h->hlines[i], "%-8.8s= %20.1f /%-50.50s",
            "CRPIX1", crpix1, "Refpix of first axis");
    i = i + 1;

    /* CRPIX2 */
    sprintf(h->hlines[i], "%-8.8s= %20.1f /%-50.50s",
            "CRPIX2", crpix2, "Refpix of second axis");
    i = i + 1;

    /* CRVAL1 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CRVAL1", crval1, "RA at Ref pix in decimal degrees");
    i = i + 1;

    /* CRVAL2 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CRVAL2", crval2, "DEC at Ref pix in decimal degrees");
    i = i + 1;

    /* CROTA1 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CROTA1", crota2, "Rotation angle of first axis (deg)");
    i = i + 1;

    /* CROTA2 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CROTA2", crota2, "Rotation angle of second axis (deg)");
    i = i + 1;
    if(abs(1. - abs(a/b)) > .01){
	 sprintf(h->hlines[i], "        Warning: %-70.70s",
		 "CROTA2 is inaccurate due to considerable skew");
	 i = i + 1;
    }
    sprintf(h->hlines[i], "%-8.8s= %20.13E, %20.13E /%-50.50s",
            "SKEW",a, b, "Measure of skew");
    i = i + 1;

    /* CDELT1 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CDELT1", cdelt1, "RA pixel step (deg)");
    i = i + 1;

    /* CDELT2 */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CDELT2", cdelt2, "DEC pixel step (deg)");
    i = i + 1;

    /* Add the CD matrix keywords to the header */
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CD1_1", cdmatx[0] , "CD matrix         ");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CD1_2", cdmatx[1] , "CD matrix         ");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CD2_1", cdmatx[2] , "CD matrix");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "CD2_2", cdmatx[3] , "CD matrix");
    i = i + 1;

    /* Now add the PC matrix keywords to the header */

    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "PC001001", pcmatx[0] , "PC matrix");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "PC001002", pcmatx[1] , "PC matrix");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "PC002001", pcmatx[2] , "PC matrix");
    i = i + 1;
    sprintf(h->hlines[i], "%-8.8s= %20.13E /%-50.50s",
            "PC002002", pcmatx[3] , "PC matrix");
    i = i + 1;

    /*
     * put last line back on the end
     */
    if (i >= h->nhlines) {
        for (j=0; j<sizeof(hlast); j++) h->hlines[i][j] = hlast[j];
        h->nhlines = i+1;
    }
}

/* ccgseq.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int ccgseq_(platea, platec, platex, platey, xpixee, ypixee, 
	platee, amdx, amdy, objecx, objecy, objecg, objecl, objeca, objecc)
double *platea, *platec, *platex, *platey, *xpixee, *ypixee, *platee, 
	*amdx, *amdy, *objecx, *objecy, *objecg, *objecl, *objeca, *objecc;
{
    /* System generated locals */
    double d__1, d__2, d__3, d__4, d__5, d__6, d__7, d__8;

    /* Local variables */
    static double x, y, p1, p2, p3, p4, etaobt, xiobjt;
    extern /* Subroutine */ int trsteq_();

    /* Parameter adjustments */
    --amdy;
    --amdx;

    /* Function Body */
    x = (*platex - *objecx * *xpixee) / 1e3;
    y = (*objecy * *ypixee - *platey) / 1e3;
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
    p1 = amdx[1] * x + amdx[2] * y + amdx[3] + amdx[4] * (d__1 * d__1) + amdx[
	    5] * x * y + amdx[6] * (d__2 * d__2);
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 3rd power */
    d__3 = x, d__4 = d__3;
/* Computing 2nd power */
    d__5 = x;
/* Computing 2nd power */
    d__6 = y;
/* Computing 3rd power */
    d__7 = y, d__8 = d__7;
    p2 = amdx[7] * (d__1 * d__1 + d__2 * d__2) + amdx[8] * (d__4 * (d__3 * 
	    d__3)) + amdx[9] * (d__5 * d__5) * y + amdx[10] * x * (d__6 * 
	    d__6) + amdx[11] * (d__8 * (d__7 * d__7));
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 2nd power */
    d__4 = x;
/* Computing 2nd power */
    d__5 = y;
/* Computing 2nd power */
    d__3 = d__4 * d__4 + d__5 * d__5;
/* Computing 2nd power */
    d__6 = *objecg;
/* Computing 3rd power */
    d__7 = *objecg, d__8 = d__7;
    p3 = amdx[12] * x * (d__1 * d__1 + d__2 * d__2) + amdx[13] * x * (d__3 * 
	    d__3) + amdx[14] * *objecg + amdx[15] * (d__6 * d__6) + amdx[16] *
	     (d__8 * (d__7 * d__7));
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 2nd power */
    d__3 = x;
/* Computing 2nd power */
    d__4 = y;
    p4 = amdx[17] * *objecg * x + amdx[18] * *objecg * (d__1 * d__1 + d__2 * 
	    d__2) + amdx[19] * *objecg * x * (d__3 * d__3 + d__4 * d__4) + 
	    amdx[20] * *objecl;
    xiobjt = p1 + p2 + p3 + p4;
/* Computing 2nd power */
    d__1 = y;
/* Computing 2nd power */
    d__2 = x;
    p1 = amdy[1] * y + amdy[2] * x + amdy[3] + amdy[4] * (d__1 * d__1) + amdy[
	    5] * x * y + amdy[6] * (d__2 * d__2);
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 3rd power */
    d__3 = y, d__4 = d__3;
/* Computing 2nd power */
    d__5 = y;
/* Computing 2nd power */
    d__6 = x;
/* Computing 3rd power */
    d__7 = x, d__8 = d__7;
    p2 = amdy[7] * (d__1 * d__1 + d__2 * d__2) + amdy[8] * (d__4 * (d__3 * 
	    d__3)) + amdy[9] * (d__5 * d__5) * x + amdy[10] * y * (d__6 * 
	    d__6) + amdy[11] * (d__8 * (d__7 * d__7));
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 2nd power */
    d__4 = x;
/* Computing 2nd power */
    d__5 = y;
/* Computing 2nd power */
    d__3 = d__4 * d__4 + d__5 * d__5;
/* Computing 2nd power */
    d__6 = *objecg;
/* Computing 3rd power */
    d__7 = *objecg, d__8 = d__7;
    p3 = amdy[12] * y * (d__1 * d__1 + d__2 * d__2) + amdy[13] * y * (d__3 * 
	    d__3) + amdy[14] * *objecg + amdy[15] * (d__6 * d__6) + amdy[16] *
	     (d__8 * (d__7 * d__7));
/* Computing 2nd power */
    d__1 = x;
/* Computing 2nd power */
    d__2 = y;
/* Computing 2nd power */
    d__3 = x;
/* Computing 2nd power */
    d__4 = y;
    p4 = amdy[17] * *objecg * y + amdy[18] * *objecg * (d__1 * d__1 + d__2 * 
	    d__2) + amdy[19] * *objecg * y * (d__3 * d__3 + d__4 * d__4) + 
	    amdy[20] * *objecl;
    etaobt = p1 + p2 + p3 + p4;
    trsteq_(platea, platec, &xiobjt, &etaobt, objeca, objecc);
/* L100: */
    return 0;
} /* ccgseq_ */

/* calcds.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/


/* Subroutine */ int calcds_(pltcea, pltcec, pltcex, pltcey, xcornr, ycornr, 
	xpixee, ypixee, platee, xsize, ysize, imcena, imcenc, amdx, amdy, 
	cdmatx)
double *pltcea, *pltcec, *pltcex, *pltcey;
float  *xcornr, *ycornr;
double *xpixee, *ypixee, *platee;
int *xsize, *ysize;
double *imcena, *imcenc, *amdx, *amdy, *cdmatx;
{
    /* System generated locals */
    int i__1, i__2, i__3, i__4;

    /* Builtin functions */
    double sqrt();

    /* Local variables */
    static double xref, yref;
    static int xlim, ylim, npts, i, j, k;
    double *u, *v, *w;
    static double x, y;
    static double color, ra, xc, yc;
    static int sp;
    extern /* Subroutine */ int ccgseq_();
    static int nx, ny, sx, sy;
    static double xcoeff[2], ycoeff[2];
    static double ww[361];
    static double xsigma[2], ysigma[2], xchisr, ychisr;
    extern /* Subroutine */ int fitsvd_(), varsvd_();
    static int nterms;
    static double newplx, newply;
    extern /* Subroutine */ int treqst_();
    static double dec, mag;
    static int nxy;
    double *eta,*cvm,*xarr,*yarr,*etap,*xi,*xip;

    /* Function Body */
    mag = 0.;
    color = 0.;
    newplx = *xsize / 2. * *xpixee;
    newply = *ysize / 2. * *ypixee;

/* Replace salloc calls with malloc using C pointers */
/* Note that 361 is the largest possible number of
   points in the grid. For small values of xsize/ysize
   this number exceeds the 100 which was allocated in 
   the original version. RNH/BP ST-ECF, March 1995 */
   xip=(double *)malloc(361*sizeof(double));
   etap=(double *)malloc(361*sizeof(double));
   xarr=(double *)malloc(361*sizeof(double)*2);
   yarr=(double *)malloc(361*sizeof(double)*2);

/* Computing MAX */
    i__1 = 1, i__2 = *xsize / 10;
    sx = max(i__1,i__2);
/* Computing MAX */
    i__1 = 1, i__2 = *ysize / 10;
    sy = max(i__1,i__2);

    xlim = *xsize - *xsize % sx;
    ylim = *ysize - *ysize % sy;

    nx = xlim / sx;
    ny = ylim / sy;
    nxy = nx * ny;
    xi = xip;
    eta = etap;
    k = 0;
    i__1 = xlim;
    i__2 = sx;
    for (i = sx; i__2 < 0 ? i >= i__1 : i <= i__1; i += i__2) {
	y = (double) i;
	i__3 = ylim;
	i__4 = sy;
	for (j = sy; i__4 < 0 ? j >= i__3 : j <= i__3; j += i__4) {
	    x = (double) j;
	    xc = x + (double)*xcornr;
	    yc = y + (double)*ycornr;
	    ccgseq_(pltcea, pltcec, pltcex, pltcey, xpixee, ypixee, platee, 
		    amdx, amdy, &xc, &yc, &mag, &color, &ra, &dec);
	    treqst_(imcena, imcenc, &ra, &dec, xi, eta);

	    ++xi;
	    ++eta;

	    xref = (x * *xpixee - newplx) / 1e3;
	    yref = (y * *ypixee - newply) / 1e3;
	    *(xarr + k) = xref;
	    *(xarr + k + nxy) = yref;
	    *(yarr + k) = yref;
	    *(yarr + k + nxy) = xref;
	    ++k;
	    ww[k-1] = 1.;
/* L120: */
	}
/* L121: */
/* L110: */
    }
/* L111: */
    npts = k;
    nterms = 2;
    u=(double *)malloc(npts*nterms*sizeof(double));
    v=(double *)malloc(nterms*nterms*sizeof(double));
    w=(double *)malloc(nterms*sizeof(double));
    cvm=(double *)malloc(nterms*nterms*sizeof(double));

    fitsvd_(xarr, xip, ww, &npts, xcoeff, &
	    nterms, u, v, w , &xchisr);
    varsvd_(v, &nterms, w, cvm, &nterms);

    for (i = 0; i < nterms; ++i) {
	xsigma[i] = sqrt(*(cvm + (i-1) + (i-1)*nterms));

/* L130: */
    }
/* L131: */
    fitsvd_(yarr, etap, ww, &npts, ycoeff, &
	    nterms, u, v, w , &ychisr);
    varsvd_(v, &nterms, w, cvm, &nterms);
    for (i = 0; i < nterms; ++i) {
	ysigma[i] = sqrt(*(cvm + (i-1) + (i-1)*nterms));
/* L140: */
    }
/* L141: */
    cdmatx[0] = xcoeff[0] * (*xpixee / 1e3) * 2.7777777777777778e-4;
    cdmatx[1] = xcoeff[1] * (*ypixee / 1e3) * 2.7777777777777778e-4;
    cdmatx[2] = ycoeff[1] * (*ypixee / 1e3) * 2.7777777777777778e-4;
    cdmatx[3] = ycoeff[0] * (*xpixee / 1e3) * 2.7777777777777778e-4;
/* L100: */

/* Free the memory */
   free(xip);
   free(etap);
   free(xarr);
   free(yarr);
   free(u);
   free(v);
   free(w);
   free(cvm);

    return 0;
} /* calcds_ */

/* trsteq.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int trsteq_(platea, platec, xi, eta, ra, dec)
double *platea, *platec, *xi, *eta, *ra, *dec;
{
    /* Builtin functions */
    double cos(), tan(), atan2();

    /* Local variables */
    static double objeca, objeci, denomr, numerr;

    objeci = *xi / 206264.8062470964;
    objeca = *eta / 206264.8062470964;
    numerr = objeci / cos(*platec);
    denomr = 1. - objeca * tan(*platec);
    *ra = atan2(numerr, denomr) + *platea;

    /* Range check */
    if( *ra < 0.0) *ra += 6.2831853071795864769;
    if( *ra > 6.2831853071795864769) *ra -= 6.2831853071795864769;
     
    numerr = cos(*ra - *platea) * (objeca + tan(*platec));
    denomr = 1. - objeca * tan(*platec);
    *dec = atan2(numerr, denomr);

    /* Added by RNH, Feb 1995 - check for declination out of range also */
    if(*dec < -3.141592653589793/2.0) *dec += 3.141592653589793;
    if(*dec > 3.141592653589793/2.0) *dec -= 3.141592653589793;

/* L100: */
    return 0;
} /* trsteq_ */

/* ksbsvd.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int ksbsvd_(u, w, v, m, n, b, x)
double *u, *w, *v;
int *m, *n;
double *b, *x;
{
    /* System generated locals */
    int u_dim1, u_offset, v_dim1, v_offset, i__1, i__2;

    /* Local variables */
    static int i, j;
    static double s;
    static int jj;
    static double tmp[1000];

    /* Parameter adjustments */
    --x;
    --b;
    v_dim1 = *n;
    v_offset = v_dim1 + 1;
    v -= v_offset;
    --w;
    u_dim1 = *m;
    u_offset = u_dim1 + 1;
    u -= u_offset;

    /* Function Body */
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	s = 0.;
	if (w[j] != 0.) {
	    i__2 = *m;
	    for (i = 1; i <= i__2; ++i) {
		s += u[i + j * u_dim1] * b[i];
	    }
	    s /= w[j];
	}
	tmp[j - 1] = s;
    }
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	s = 0.;
	i__2 = *n;
	for (jj = 1; jj <= i__2; ++jj) {
	    s += v[j + jj * v_dim1] * tmp[jj - 1];
	}
	x[j] = s;
    }
    return 0;
} /* ksbsvd_ */

/* varsvd.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int varsvd_(v, ma, w, cvm, ncvm)
double *v;
int *ma;
double *w, *cvm;
int *ncvm;
{
    /* System generated locals */
    int v_dim1, v_offset, cvm_dim1, cvm_offset, i__1, i__2, i__3;

    /* Local variables */
    static int i, j, k;
    static double wti[20], sum;

    /* Parameter adjustments */
    cvm_dim1 = *ncvm;
    cvm_offset = cvm_dim1 + 1;
    cvm -= cvm_offset;
    --w;
    v_dim1 = *ma;
    v_offset = v_dim1 + 1;
    v -= v_offset;

    /* Function Body */
    i__1 = *ma;
    for (i = 1; i <= i__1; ++i) {
	wti[i - 1] = 0.;
	if (w[i] != 0.) {
	    wti[i - 1] = 1. / (w[i] * w[i]);
	}
    }
    i__1 = *ma;
    for (i = 1; i <= i__1; ++i) {
	i__2 = i;
	for (j = 1; j <= i__2; ++j) {
	    sum = 0.;
	    i__3 = *ma;
	    for (k = 1; k <= i__3; ++k) {
		sum += v[i + k * v_dim1] * v[j + k * v_dim1] * wti[k - 1];
	    }
	    cvm[i + j * cvm_dim1] = sum;
	    cvm[j + i * cvm_dim1] = sum;
	}
    }
    return 0;
} /* varsvd_ */

/* dcmpsv.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int dcmpsv_(a, m, n, w, v)
double *a;
int *m, *n;
double *w, *v;
{
    /* System generated locals */
    int a_dim1, a_offset, v_dim1, v_offset, i__1, i__2, i__3;
    double d__1, d__2, d__3, d__4;

    /* Builtin functions */
    double sqrt(); 
    double d_sign();

    /* Local variables */
    static double c, f, g, h;
    static int i, j, k, l;
    static double s, x, y, z, scale, anorm;
    static int nm;
    static double rv1[1000];
    static int its;

    /* Parameter adjustments */
    v_dim1 = *n;
    v_offset = v_dim1 + 1;
    v -= v_offset;
    --w;
    a_dim1 = *m;
    a_offset = a_dim1 + 1;
    a -= a_offset;

    /* Function Body */
    g = 0.;
    scale = 0.;
    anorm = 0.;
    i__1 = *n;
    for (i = 1; i <= i__1; ++i) {
	l = i + 1;
	rv1[i - 1] = scale * g;
	g = 0.;
	s = 0.;
	scale = 0.;
	if (i <= *m) {
	    i__2 = *m;
	    for (k = i; k <= i__2; ++k) {
		scale += (d__1 = a[k + i * a_dim1], abs(d__1));
	    }
	    if (scale != 0.) {
		i__2 = *m;
		for (k = i; k <= i__2; ++k) {
		    a[k + i * a_dim1] /= scale;
		    s += a[k + i * a_dim1] * a[k + i * a_dim1];
		}
		f = a[i + i * a_dim1];
		d__1 = sqrt(s);
		g = -d_sign(&d__1, &f);
		h = f * g - s;
		a[i + i * a_dim1] = f - g;
		if (i != *n) {
		    i__2 = *n;
		    for (j = l; j <= i__2; ++j) {
			s = 0.;
			i__3 = *m;
			for (k = i; k <= i__3; ++k) {
			    s += a[k + i * a_dim1] * a[k + j * a_dim1];
			}
			f = s / h;
			i__3 = *m;
			for (k = i; k <= i__3; ++k) {
			    a[k + j * a_dim1] += f * a[k + i * a_dim1];
			}
		    }
		}
		i__2 = *m;
		for (k = i; k <= i__2; ++k) {
		    a[k + i * a_dim1] = scale * a[k + i * a_dim1];
		}
	    }
	}
	w[i] = scale * g;
	g = 0.;
	s = 0.;
	scale = 0.;
	if (i <= *m && i != *n) {
	    i__2 = *n;
	    for (k = l; k <= i__2; ++k) {
		scale += (d__1 = a[i + k * a_dim1], abs(d__1));
	    }
	    if (scale != 0.) {
		i__2 = *n;
		for (k = l; k <= i__2; ++k) {
		    a[i + k * a_dim1] /= scale;
		    s += a[i + k * a_dim1] * a[i + k * a_dim1];
		}
		f = a[i + l * a_dim1];
		d__1 = sqrt(s);
		g = -d_sign(&d__1, &f);
		h = f * g - s;
		a[i + l * a_dim1] = f - g;
		i__2 = *n;
		for (k = l; k <= i__2; ++k) {
		    rv1[k - 1] = a[i + k * a_dim1] / h;
		}
		if (i != *m) {
		    i__2 = *m;
		    for (j = l; j <= i__2; ++j) {
			s = 0.;
			i__3 = *n;
			for (k = l; k <= i__3; ++k) {
			    s += a[j + k * a_dim1] * a[i + k * a_dim1];
			}
			i__3 = *n;
			for (k = l; k <= i__3; ++k) {
			    a[j + k * a_dim1] += s * rv1[k - 1];
			}
		    }
		}
		i__2 = *n;
		for (k = l; k <= i__2; ++k) {
		    a[i + k * a_dim1] = scale * a[i + k * a_dim1];
		}
	    }
	}
/* Computing MAX */
	d__3 = anorm, d__4 = (d__1 = w[i], abs(d__1)) + (d__2 = rv1[i - 1], 
		abs(d__2));
	anorm = max(d__3,d__4);
    }
    for (i = *n; i >= 1; --i) {
	if (i < *n) {
	    if (g != 0.) {
		i__1 = *n;
		for (j = l; j <= i__1; ++j) {
		    v[j + i * v_dim1] = a[i + j * a_dim1] / a[i + l * a_dim1] 
			    / g;
		}
		i__1 = *n;
		for (j = l; j <= i__1; ++j) {
		    s = 0.;
		    i__2 = *n;
		    for (k = l; k <= i__2; ++k) {
			s += a[i + k * a_dim1] * v[k + j * v_dim1];
		    }
		    i__2 = *n;
		    for (k = l; k <= i__2; ++k) {
			v[k + j * v_dim1] += s * v[k + i * v_dim1];
		    }
		}
	    }
	    i__1 = *n;
	    for (j = l; j <= i__1; ++j) {
		v[i + j * v_dim1] = 0.;
		v[j + i * v_dim1] = 0.;
	    }
	}
	v[i + i * v_dim1] = 1.;
	g = rv1[i - 1];
	l = i;
    }
    for (i = *n; i >= 1; --i) {
	l = i + 1;
	g = w[i];
	if (i < *n) {
	    i__1 = *n;
	    for (j = l; j <= i__1; ++j) {
		a[i + j * a_dim1] = 0.;
	    }
	}
	if (g != 0.) {
	    g = 1. / g;
	    if (i != *n) {
		i__1 = *n;
		for (j = l; j <= i__1; ++j) {
		    s = 0.;
		    i__2 = *m;
		    for (k = l; k <= i__2; ++k) {
			s += a[k + i * a_dim1] * a[k + j * a_dim1];
		    }
		    f = s / a[i + i * a_dim1] * g;
		    i__2 = *m;
		    for (k = i; k <= i__2; ++k) {
			a[k + j * a_dim1] += f * a[k + i * a_dim1];
		    }
		}
	    }
	    i__1 = *m;
	    for (j = i; j <= i__1; ++j) {
		a[j + i * a_dim1] *= g;
	    }
	} else {
	    i__1 = *m;
	    for (j = i; j <= i__1; ++j) {
		a[j + i * a_dim1] = 0.;
	    }
	}
	a[i + i * a_dim1] += 1.;
    }
    for (k = *n; k >= 1; --k) {
	for (its = 1; its <= 30; ++its) {
	    for (l = k; l >= 1; --l) {
		nm = l - 1;
		if ((d__1 = rv1[l - 1], abs(d__1)) + anorm == anorm) {
		    goto L2;
		}
		if ((d__1 = w[nm], abs(d__1)) + anorm == anorm) {
		    goto L1;
		}
	    }
L1:
	    c = 0.;
	    s = 1.;
	    i__1 = k;
	    for (i = l; i <= i__1; ++i) {
		f = s * rv1[i - 1];
		if (abs(f) + anorm != anorm) {
		    g = w[i];
		    h = sqrt(f * f + g * g);
		    w[i] = h;
		    h = 1. / h;
		    c = g * h;
		    s = -(f * h);
		    i__2 = *m;
		    for (j = 1; j <= i__2; ++j) {
			y = a[j + nm * a_dim1];
			z = a[j + i * a_dim1];
			a[j + nm * a_dim1] = y * c + z * s;
			a[j + i * a_dim1] = -(y * s) + z * c;
		    }
		}
	    }
L2:
	    z = w[k];
	    if (l == k) {
		if (z < 0.) {
		    w[k] = -z;
		    i__1 = *n;
		    for (j = 1; j <= i__1; ++j) {
			v[j + k * v_dim1] = -v[j + k * v_dim1];
		    }
		}
		goto L3;
	    }
	    if (its == 30) {
		printf("No convergence in 30 iterations\n");
	    }
	    x = w[l];
	    nm = k - 1;
	    y = w[nm];
	    g = rv1[nm - 1];
	    h = rv1[k - 1];
	    f = ((y - z) * (y + z) + (g - h) * (g + h)) / (h * 2. * y);
	    g = sqrt(f * f + 1.);
	    f = ((x - z) * (x + z) + h * (y / (f + d_sign(&g, &f)) - h)) / x;
	    c = 1.;
	    s = 1.;
	    i__1 = nm;
	    for (j = l; j <= i__1; ++j) {
		i = j + 1;
		g = rv1[i - 1];
		y = w[i];
		h = s * g;
		g = c * g;
		z = sqrt(f * f + h * h);
		rv1[j - 1] = z;
		c = f / z;
		s = h / z;
		f = x * c + g * s;
		g = -(x * s) + g * c;
		h = y * s;
		y *= c;
		i__2 = *n;
		for (nm = 1; nm <= i__2; ++nm) {
		    x = v[nm + j * v_dim1];
		    z = v[nm + i * v_dim1];
		    v[nm + j * v_dim1] = x * c + z * s;
		    v[nm + i * v_dim1] = -(x * s) + z * c;
		}
		z = sqrt(f * f + h * h);
		w[j] = z;
		if (z != 0.) {
		    z = 1. / z;
		    c = f * z;
		    s = h * z;
		}
		f = c * g + s * y;
		x = -(s * g) + c * y;
		i__2 = *m;
		for (nm = 1; nm <= i__2; ++nm) {
		    y = a[nm + j * a_dim1];
		    z = a[nm + i * a_dim1];
		    a[nm + j * a_dim1] = y * c + z * s;
		    a[nm + i * a_dim1] = -(y * s) + z * c;
		}
	    }
	    rv1[l - 1] = 0.;
	    rv1[k - 1] = f;
	    w[k] = x;
	}
L3:
	;
    }
    return 0;
} /* dcmpsv_ */

/* fitsvd.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int fitsvd_(x, y, wg, npts, coef, nterms, u, v, w, chisq)
double *x, *y;
double *wg;
int *npts;
double *coef;
int *nterms;
double *u, *v, *w, *chisq;
{
    /* System generated locals */
    int x_dim1, x_offset, v_dim1, v_offset, u_dim1, u_offset, i__1, i__2;
    double d__1;

    /* Local variables */
    static double wmax, b[1000];
    static int i, j;
    extern /* Subroutine */ int dcmpsv_();
    static double thresh;
    extern /* Subroutine */ int ksbsvd_();
    static double sum;

    /* Parameter adjustments */
    --w;
    v_dim1 = *nterms;
    v_offset = v_dim1 + 1;
    v -= v_offset;
    u_dim1 = *npts;
    u_offset = u_dim1 + 1;
    u -= u_offset;
    --coef;
    --wg;
    --y;
    x_dim1 = *npts;
    x_offset = x_dim1 + 1;
    x -= x_offset;

    /* Function Body */
    i__1 = *npts;
    for (i = 1; i <= i__1; ++i) {
	i__2 = *nterms;
	for (j = 1; j <= i__2; ++j) {
	    u[i + j * u_dim1] = x[i + j * x_dim1] * wg[i];
	}
	b[i - 1] = y[i] * wg[i];
    }
    dcmpsv_(&u[u_offset], npts, nterms, &w[1], &v[v_offset]);
    wmax = 0.;
    i__1 = *nterms;
    for (j = 1; j <= i__1; ++j) {
	if (w[j] > wmax) {
	    wmax = w[j];
	}
    }
    thresh = wmax * 1e-14;
    i__1 = *nterms;
    for (j = 1; j <= i__1; ++j) {
	if (w[j] < thresh) {
	    w[j] = 0.;
	}
    }
    ksbsvd_(&u[u_offset], &w[1], &v[v_offset], npts, nterms, b, &coef[1]);
    *chisq = 0.;
    i__1 = *npts;
    for (i = 1; i <= i__1; ++i) {
	sum = 0.;
	i__2 = *nterms;
	for (j = 1; j <= i__2; ++j) {
	    sum += coef[j] * x[i + j * x_dim1];
	}
/* Computing 2nd power */
	d__1 = (y[i] - sum) * wg[i];
	*chisq += d__1 * d__1;
    }
    return 0;
} /* fitsvd_ */

/* treqst.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* Subroutine */ int treqst_(platea, platec, objeca, objecc, xiobjt, etaobt)
double *platea, *platec, *objeca, *objecc, *xiobjt, *etaobt;
{
    /* Builtin functions */
    double sin(), cos();

    /* Local variables */
    static double div;

    div = sin(*objecc) * sin(*platec) + cos(*objecc) * cos(*platec) * cos(*
	    objeca - *platea);
    *xiobjt = cos(*objecc) * sin(*objeca - *platea) * 206264.8062470964 / div;
    *etaobt = (sin(*objecc) * cos(*platec) - cos(*objecc) * sin(*platec) * 
	    cos(*objeca - *platea)) * 206264.8062470964 / div;
/* L100: */
    return 0;
} /* treqst_ */

double d_sign(a,b)   

/* This routine is available in SUNOS 4.1.3 but not in Solaris
   and possibly other system. So we provide our own version. */

   double *a,*b;
   {
   if (*b >= 0.) return(fabs(*a));
   else return(-fabs(*a));
   }

#if 0
/* regren.f -- translated by f2c (version of 23 April 1993  18:34:30).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

/* The following is the IDL version of the regression routine by Bevington */
/*Some modification have been made to the book version in regard to the input
*/
/* X array and the data type of some of the arrays. */
/* FUNCTION REGRESS,X,Y,W,YFIT,A0,SIGMA,FTEST,R,RMUL,CHISQ */
/* ; */
/* ;+ */
/* ; NAME: */
/* ;	REGRESS */
/* ; PURPOSE: */
/* ;	Multiple linear regression fit. */
/* ;	Fit the function: */
/* ;	Y(i) = A0 + A(0)*X(0,i) + A(1)*X(1,i) + ... + */
/* ;		A(Nterms-1)*X(Nterms-1,i) */
/* ; CATEGORY: */
/* ;	G2 - Correlation and regression analysis. */
/* ; CALLING SEQUENCE: */
/* ;	Coeff = REGRESS(X,Y,W,YFIT,A0,SIGMA,FTEST,RMUL,CHISQ) */
/* ; INPUTS: */
/* ;	X = array of independent variable data.  X must */
/* ;		be dimensioned (Nterms, Npoints) where there are Nterms */
/* ;		coefficients to be found (independent variables) and */
/* ;		Npoints of samples. */
/* ;	Y = vector of dependent variable points, must */
/* ;		have Npoints elements. */
/* ;	W = vector of weights for each equation, must */
/* ;		be a Npoints elements vector.  For no */
/* ;		weighting, set w(i) = 1., for instrumental weighting */
/* ;		w(i) = 1/standard_deviation(Y(i)), for statistical */
/* ;		weighting w(i) = 1./Y(i) */
/* ; */
/* ; OUTPUTS: */
/* ;	Function result = coefficients = vector of */
/* ;		Nterms elements.  Returned as a column */
/* ;		vector. */
/* ; */
/* ; OPTIONAL OUTPUT PARAMETERS: */
/* ;	Yfit = array of calculated values of Y, Npoints */
/* ;		elements. */
/* ;	A0 = Constant term. */
/* ;	Sigma = Vector of standard deviations for */
/* ;		coefficients. */
/* ;	Ftest = value of F for test of fit. */
/* ;	Rmul = multiple linear correlation coefficient. */
/* ;	R = Vector of linear correlation coefficient. */
/* ;	Chisq = Reduced weighted chi squared. */
/* ; COMMON BLOCKS: */
/* ;	None. */
/* ; SIDE EFFECTS: */
/* ;	None. */
/* ; RESTRICTIONS: */
/* ;	None. */
/* ; PROCEDURE: */
/* ;	Adapted from the program REGRES, Page 172, */
/* ;		Bevington, Data Reduction and Error Analysis for the */
/* ;		Physical Sciences, 1969. */
/* ; */
/* ; MODIFICATION HISTORY: */
/* ;	Written, DMS, RSI, September, 1982. */
/* ;- */
/* ; */
/* SY = SIZE(Y)		;GET DIMENSIONS OF X AND Y. */
/* SX = SIZE(X) */
/*IF (N_ELEMENTS(W) NE SY(1)) OR (SX(0) NE 2) OR (SY(1) NE SX(2)) THEN BEGIN*/
/* 	PRINT,'REGRESS - Incompatible arrays' */
/* 	RETURN,0 */
/* 	ENDIF */
/* ; */
/* NTERM = SX(1)		;# OF TERMS */
/* NPTS = SY(1)		;# OF OBSERVATIONS */
/* ; */
/* SW = TOTAL(W)		;SUM OF WEIGHTS */
/* YMEAN = TOTAL(Y*W)/SW	;Y MEAN */
/* XMEAN = (X * (REPLICATE(1.,NTERM) # W)) # REPLICATE(1./SW,NPTS) */
/* WMEAN = SW/NPTS */
/* WW = W/WMEAN */
/* ; */
/* NFREE = NPTS-1		;DEGS OF FREEDOM */
/* SIGMAY = SQRT(TOTAL(WW * (Y-YMEAN)^2)/NFREE) ;W*(Y(I)-YMEAN) */
/* XX = X- XMEAN # REPLICATE(1.,NPTS)	;X(J,I) - XMEAN(I) */
/* WX = REPLICATE(1.,NTERM) # WW * XX	;W(I)*(X(J,I)-XMEAN(I)) */
/*SIGMAX = SQRT( XX*WX # REPLICATE(1./NFREE,NPTS)) ;W(I)*(X(J,I)-XM)*(X(K,I)-X
M)*/
/* R = WX #(Y - YMEAN) / (SIGMAX * SIGMAY * NFREE) */
/* ARRAY = INVERT((WX # TRANSPOSE(XX))/(NFREE * SIGMAX #SIGMAX)) */
/* A = (R # ARRAY)*(SIGMAY/SIGMAX)		;GET COEFFICIENTS */
/* YFIT = A # X				;COMPUTE FIT */
/* A0 = YMEAN - TOTAL(A*XMEAN)		;CONSTANT TERM */
/* YFIT = YFIT + A0			;ADD IT IN */
/* FREEN = NPTS-NTERM-1 > 1		;DEGS OF FREEDOM, AT LEAST 1. */
/* CHISQ = TOTAL(WW*(Y-YFIT)^2)*WMEAN/FREEN ;WEIGHTED CHI SQUARED */
/*SIGMA = SQRT(ARRAY(INDGEN(NTERM)*(NTERM+1))/WMEAN/(NFREE*SIGMAX^2)) ;ERROR T
ERM*/
/* RMUL = TOTAL(A*R*SIGMAX/SIGMAY)		;MULTIPLE LIN REG COEFF */
/*IF RMUL LT 1. THEN FTEST = RMUL/NTERM / ((1.-RMUL)/FREEN) ELSE FTEST=1.E6*/
/* RMUL = SQRT(RMUL) */
/* RETURN,A */
/* END */
/* Subroutine */ int regren_(x, ndim1, ndim2, y, weight, npts, nterms, yfit, 
	a0, a, sigmaa, chisqr)
double *x;
int *ndim1, *ndim2;
double *y;
float *weight;
int *npts, *nterms;
double *yfit, *a0, *a;
float *sigmaa;
double *chisqr;
{
    /* System generated locals */
    int x_dim1, x_offset, i__1, i__2, i__3;
    double d__1;

    /* Builtin functions */
    double sqrt();

    /* Local variables */
    static float rmul, free1;
    static int i, j, k;
    static double r[20];
    static float freej, freen;
    static double sigma, chisq, xmean[20], ymean;
    static float wmean;
    extern /* Subroutine */ int minv20_();
    static double array[400]	/* was [20][20] */;
    static float ftest, fnpts, sigma0, varnce;
    static double sigmax[20];
    static float det;
    static double sum;


/* initialize sums and arrays */

    /* Parameter adjustments */
    --sigmaa;
    --a;
    --yfit;
    --weight;
    --y;
    x_dim1 = *ndim1;
    x_offset = x_dim1 + 1;
    x -= x_offset;

    /* Function Body */
/* L11: */
    sum = (float)0.;
    ymean = (float)0.;
    sigma = (float)0.;
    chisq = (float)0.;
    rmul = (float)0.;
    i__1 = *npts;
    for (i = 1; i <= i__1; ++i) {
/* L17: */
	yfit[i] = (float)0.;
    }
/* L21: */
    i__1 = *nterms;
    for (j = 1; j <= i__1; ++j) {
	xmean[j - 1] = (float)0.;
	sigmax[j - 1] = (float)0.;
	r[j - 1] = (float)0.;
	a[j] = (float)0.;
	sigmaa[j] = (float)0.;
	i__2 = *nterms;
	for (k = 1; k <= i__2; ++k) {
/* L28: */
	    array[j + k * 20 - 21] = (float)0.;
	}
    }

/* accumulate weighted sums */

/* L30: */
    i__2 = *npts;
    for (i = 1; i <= i__2; ++i) {
	sum += weight[i];
	ymean += weight[i] * y[i];
	i__1 = *nterms;
	for (j = 1; j <= i__1; ++j) {
/* L44: */
	    xmean[j - 1] += weight[i] * x[j + i * x_dim1];
	}
/* L50: */
    }
/* L51: */
    ymean /= sum;
    i__2 = *nterms;
    for (j = 1; j <= i__2; ++j) {
/* L53: */
	xmean[j - 1] /= sum;
    }
    fnpts = (float) (*npts);
    wmean = sum / fnpts;
    i__2 = *npts;
    for (i = 1; i <= i__2; ++i) {
/* L57: */
	weight[i] /= wmean;
    }

/* accumulate matrices r and array */

/* L61: */
    i__2 = *npts;
    for (i = 1; i <= i__2; ++i) {
/* Computing 2nd power */
	d__1 = y[i] - ymean;
	sigma += weight[i] * (d__1 * d__1);
	i__1 = *nterms;
	for (j = 1; j <= i__1; ++j) {
/* Computing 2nd power */
	    d__1 = x[j + i * x_dim1] - xmean[j - 1];
	    sigmax[j - 1] += weight[i] * (d__1 * d__1);
	    r[j - 1] += weight[i] * (x[j + i * x_dim1] - xmean[j - 1]) * (y[i]
		     - ymean);
	    i__3 = j;
	    for (k = 1; k <= i__3; ++k) {
/* L67: */
		array[j + k * 20 - 21] += weight[i] * (x[j + i * x_dim1] - 
			xmean[j - 1]) * (x[k + i * x_dim1] - xmean[k - 1]);
	    }
	}
    }
/* L71: */
    free1 = (float) (*npts - 1);
/* L72: */
    sigma = sqrt(sigma / free1);
    i__3 = *nterms;
    for (j = 1; j <= i__3; ++j) {
/* L74: */
	sigmax[j - 1] = sqrt(sigmax[j - 1] / free1);
	r[j - 1] /= free1 * sigmax[j - 1] * sigma;
	i__1 = j;
	for (k = 1; k <= i__1; ++k) {
	    array[j + k * 20 - 21] /= free1 * sigmax[j - 1] * sigmax[k - 1];
/* L78: */
	    array[k + j * 20 - 21] = array[j + k * 20 - 21];
	}
    }

/* invert symmetric matrix */

/* L81: */
    minv20_(array, nterms, &det);
    if (det != (float)0.) {
	goto L101;
    } else {
	goto L91;
    }
L91:
    *a0 = (float)0.;
    sigma0 = (float)0.;
    rmul = (float)0.;
    *chisqr = (float)0.;
    ftest = (float)0.;
    goto L150;

/* calculate coefficients, fit, and chi square */

L101:
    *a0 = ymean;
/* L102: */
    i__1 = *nterms;
    for (j = 1; j <= i__1; ++j) {
	i__3 = *nterms;
	for (k = 1; k <= i__3; ++k) {
/* L104: */
	    a[j] += r[k - 1] * array[j + k * 20 - 21];
	}
/* L105: */
	a[j] = a[j] * sigma / sigmax[j - 1];
/* L106: */
	*a0 -= a[j] * xmean[j - 1];
/* L107: */
	i__3 = *npts;
	for (i = 1; i <= i__3; ++i) {
/* L108: */
	    yfit[i] += a[j] * x[j + i * x_dim1];
	}
    }
/* L111: */
    i__3 = *npts;
    for (i = 1; i <= i__3; ++i) {
	yfit[i] += *a0;
/* L113: */
/* Computing 2nd power */
	d__1 = y[i] - yfit[i];
	chisq += weight[i] * (d__1 * d__1);
    }
    freen = (float) (*npts - *nterms - 1);
/* L115: */
    *chisqr = chisq * wmean / freen;

/* calculate uncertainties */

/* L124: */
    varnce = *chisqr;
/* L131: */
    i__3 = *nterms;
    for (j = 1; j <= i__3; ++j) {
/* L132: */
/* Computing 2nd power */
	d__1 = sigmax[j - 1];
	sigmaa[j] = array[j + j * 20 - 21] * varnce / (free1 * (d__1 * d__1));
	sigmaa[j] = sqrt(sigmaa[j]);
/* L133: */
	rmul += a[j] * r[j - 1] * sigmax[j - 1] / sigma;
    }
    freej = (float) (*nterms);
/* +noao: When rmul = 1, the following division (stmt 135) would blow up. 
*/
/*        It has been changed so ftest is set to -99999. in this case. */
    if ((float)1. - rmul != (float)0.) {
	goto L135;
    } else {
	goto L935;
    }
L135:
    ftest = rmul / freej / (((float)1. - rmul) / freen);
    goto L136;
L935:
    ftest = (float)-99999.;
/* -noao */
L136:
    rmul = sqrt(rmul);
/* L141: */
    sigma0 = varnce / fnpts;
    i__3 = *nterms;
    for (j = 1; j <= i__3; ++j) {
	i__1 = *nterms;
	for (k = 1; k <= i__1; ++k) {
/* L145: */
	    sigma0 += varnce * xmean[j - 1] * xmean[k - 1] * array[j + k * 20 
		    - 21] / (free1 * sigmax[j - 1] * sigmax[k - 1]);
	}
    }
/* L146: */
    sigma0 = sqrt(sigma0);
L150:
    return 0;
} /* regren_ */

/* subroutine matinv.f */

/* source */
/*   bevington, pages 302-303. */

/* purpose */
/*   invert a symmetric matrix and calculate its determinant */

/* usage */
/*   call matinv (array, norder, det) */

/* description of parameters */
/*   array  - input matrix which is replaced by its inverse */
/*   norder - degree of matrix (order of determinant) */
/*   det    - determinant of input matrix */

/* subroutines and function subprograms required */
/*   none */

/* comment */
/*   dimension statement valid for norder up to 20 */

/* Subroutine */ int minv20_(array, norder, det)
double *array;
int *norder;
float *det;
{
    /* System generated locals */
    int i__1, i__2, i__3;
    double d__1;

    /* Local variables */
    static double amax, save;
    static int i, j, k, l, ik[20], jk[20];


    /* Parameter adjustments */
    array -= 21;

    /* Function Body */
/* L10: */
    *det = (float)1.;
/* L11: */
    i__1 = *norder;
    for (k = 1; k <= i__1; ++k) {

/* find largest element array(i,j) in rest of matrix */

	amax = 0.;
L21:
	i__2 = *norder;
	for (i = k; i <= i__2; ++i) {
	    i__3 = *norder;
	    for (j = k; j <= i__3; ++j) {
/* L23: */
		if (abs(amax) - (d__1 = array[i + j * 20], abs(d__1)) <= 0.) {
		    goto L24;
		} else {
		    goto L30;
		}
L24:
		amax = array[i + j * 20];
		ik[k - 1] = i;
		jk[k - 1] = j;
L30:
		;
	    }
	}

/* interchange rows and columns to put amax in array(k,k) */

/* L31: */
	if (amax != 0.) {
	    goto L41;
	} else {
	    goto L32;
	}
L32:
	*det = 0.;
	goto L140;
L41:
	i = ik[k - 1];
	if ((i__3 = i - k) < 0) {
	    goto L21;
	} else if (i__3 == 0) {
	    goto L51;
	} else {
	    goto L43;
	}
L43:
	i__3 = *norder;
	for (j = 1; j <= i__3; ++j) {
	    save = array[k + j * 20];
	    array[k + j * 20] = array[i + j * 20];
/* L50: */
	    array[i + j * 20] = -save;
	}
L51:
	j = jk[k - 1];
	if ((i__3 = j - k) < 0) {
	    goto L21;
	} else if (i__3 == 0) {
	    goto L61;
	} else {
	    goto L53;
	}
L53:
	i__3 = *norder;
	for (i = 1; i <= i__3; ++i) {
	    save = array[i + k * 20];
	    array[i + k * 20] = array[i + j * 20];
/* L60: */
	    array[i + j * 20] = -save;
	}

/* accumulate elements of inverse matrix */

L61:
	i__3 = *norder;
	for (i = 1; i <= i__3; ++i) {
	    if (i - k != 0) {
		goto L63;
	    } else {
		goto L70;
	    }
L63:
	    array[i + k * 20] = -array[i + k * 20] / amax;
L70:
	    ;
	}
/* L71: */
	i__3 = *norder;
	for (i = 1; i <= i__3; ++i) {
	    i__2 = *norder;
	    for (j = 1; j <= i__2; ++j) {
		if (i - k != 0) {
		    goto L74;
		} else {
		    goto L80;
		}
L74:
		if (j - k != 0) {
		    goto L75;
		} else {
		    goto L80;
		}
L75:
		array[i + j * 20] += array[i + k * 20] * array[k + j * 20];
L80:
		;
	    }
	}
/* L81: */
	i__2 = *norder;
	for (j = 1; j <= i__2; ++j) {
	    if (j - k != 0) {
		goto L83;
	    } else {
		goto L90;
	    }
L83:
	    array[k + j * 20] /= amax;
L90:
	    ;
	}
	array[k + k * 20] = 1. / amax;
/* L100: */
	*det *= amax;
    }

/* restore ordering of matrix */

/* L101: */
    i__1 = *norder;
    for (l = 1; l <= i__1; ++l) {
	k = *norder - l + 1;
	j = ik[k - 1];
	if (j - k <= 0) {
	    goto L111;
	} else {
	    goto L105;
	}
L105:
	i__2 = *norder;
	for (i = 1; i <= i__2; ++i) {
	    save = array[i + k * 20];
	    array[i + k * 20] = -array[i + j * 20];
/* L110: */
	    array[i + j * 20] = save;
	}
L111:
	i = jk[k - 1];
	if (i - k <= 0) {
	    goto L130;
	} else {
	    goto L113;
	}
L113:
	i__2 = *norder;
	for (j = 1; j <= i__2; ++j) {
	    save = array[k + j * 20];
	    array[k + j * 20] = -array[i + j * 20];
/* L120: */
	    array[i + j * 20] = save;
	}
L130:
	;
    }
L140:
    return 0;
} /* minv20_ */
#endif
