/******************************************************************************
* C Source: Image Section                                                     
*                                                                             
* File:                                                                       
*    imgsectn.c                                                               
*                                                                             
* Description:                                                                
*    This set of routines manipulate a structure (see imgsectn.h) which       
*    contains data about a  section of plate image, namely, Plate data (see   
*    plate.c and plate.h), the center x,y pixel position of the section on    
*    on the plate, the width and height in pixels, and the distance of the    
*    section center from the plate center in pixels.                          
*                                                                             
* Module List:                                                                
*    Get_Image_Section                                                        
*       Computes data describing an image section given the requested field   
*       on a given plate and the directory where the plate image headers      
*       reside.                                                               
*    Delete_Image_Section                                                     
*       Free memory for an image section.                                     
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 29-NOV-95 by J. Doggett,                                    
*       Updated Get_Image_Section function.                                   
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
* Copyright (c) 1993-1996, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include "imgsectn.h"

#include <stdio.h>
#include <stdlib.h>

#include "astrmcal.h"
#include "astronmy.h"
#include "field.h"
#include "generic.h"
#include "header.h"
#include "plate.h"

/******************************************************************************
* Module:                                                                     
*    Get_Image_Section                                                        
*                                                                             
* File:                                                                       
*    imgsectn.c                                                               
*                                                                             
* Description:                                                                
*    Computes the x-y specifications of an image section from a requested     
*    field on a give plate.                                                   
*                                                                             
* Return Value:                                                               
*    IMAGE_SECTION: The IMAGE_SECTION for the requested field on the given    
*       plate.                                                                
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Field                                                                 
*          FIELD, Reference                                                   
*             The field for which we want an image section.                   
*       Plate                                                                 
*          PLATE, Reference                                                   
*             The plate from which we're getting an image section.            
*       Header                                                                
*          HEADER, Reference                                                  
*             Contains data in the plate header file.                         
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 29-NOV-95 by J. Doggett,                                    
*       Modified to put center, edge distances and flag that indicates the    
*       extraction is fully contained in the area of the plate into the image 
*       section structure.                                                    
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
******************************************************************************/
/*********************************
* Get_Image_Section declarations *
*********************************/
IMAGE_SECTION Get_Image_Section( Field, Plate, Header )
FIELD         Field ;
PLATE         Plate ;
HEADER        Header;
{
   IMAGE_SECTION New_Image_Section; /* Newly created image section to return */

   float Dummy_Magnitude = 0.; /* Dummy variables for the xypos      */
   float Dummy_Color     = 0.; /* function                           */
   float Field_Center_X ; /* Center X and Y pixel position of a */
   float Field_Center_Y ; /* field we're extracting.            */

   int imageSectionLeftEdge  ; /* Position of each each of extraction */

/* *** Get_Image_Section ***                                               2 */


   int imageSectionBottomEdge; /* on the plate */
   int imageSectionRightEdge ;
   int imageSectionTopEdge   ;

   int leftEdgeDistance  ; /* Distance of image section edges from the */
   int bottomEdgeDistance; /* edge of the plate */
   int rightEdgeDistance ;
   int topEdgeDistance   ;

   int i; /* For loop index */

   double centerDistance;
              /* Distance of field center to plate center in radians */

/******************************************
* Get_Image_Section begin executable code *
******************************************/

/* Figure out where field is on the plate */
   xypos( Header,
           Field->Center_RA, Field->Center_Dec,
           Dummy_Magnitude, Dummy_Color,
          &Field_Center_X, &Field_Center_Y );

/* If the field is on the plate, then set up an image section for it */
   if ( ( Field_Center_X >= 0 )
                 &&
        ( Field_Center_X <= Header->section_x_length )
                 &&
        ( Field_Center_Y >= 0 )
                 &&
        ( Field_Center_Y <= Header->section_y_length ) )
   {
      New_Image_Section =
              ( IMAGE_SECTION )Allocate_Memory( sizeof( *New_Image_Section ) );
      New_Image_Section->X     = ( int )Field_Center_X;
      New_Image_Section->Y     = ( int )Field_Center_Y;
      New_Image_Section->Width = ( int )(
             Field->Width                 /* From arcminutes*/
            *60.                          /* To arcseconds  */
            /Header->plt_scale            /* To mm          */
            *1000.                        /* To microns     */
            /Header->x_pixel_size + 0.5); /* To pixels      */
      New_Image_Section->Height = ( int )(
             Field->Height                /* From arcminutes*/
            *60.                          /* To arcseconds  */
            /Header->plt_scale            /* To mm          */
            *1000.                        /* To microns     */
            /Header->y_pixel_size + 0.5); /* To pixels      */

      centerDistance = Great_Circle_Separation(
                                          Header->plt_center_ra ,
                                          Header->plt_center_dec,
                                          Field->Center_RA     ,
                                          Field->Center_Dec
                                        );

      New_Image_Section->centerDistance = ( int )
            ( centerDistance        /* From radians  */
             *ARCSECONDS_PER_RADIAN /* To arcseconds */
             /Header->plt_scale     /* To mm         */
             *1000.                 /* To microns    */

/* *** Get_Image_Section ***                                               3 */


             /Header->y_pixel_size  /* To pixels     */
             + 0.5 );

      imageSectionLeftEdge    =
            New_Image_Section->X - New_Image_Section->Width/2.;
      imageSectionBottomEdge  =
            New_Image_Section->Y - New_Image_Section->Height/2.;
      imageSectionRightEdge   =
            imageSectionLeftEdge   + New_Image_Section->Width  - 1;
      imageSectionTopEdge     =
            imageSectionBottomEdge + New_Image_Section->Height - 1;

      leftEdgeDistance   = imageSectionLeftEdge - 1;
      New_Image_Section->edgeDistance = leftEdgeDistance;

      bottomEdgeDistance = imageSectionBottomEdge - 1;
      if (bottomEdgeDistance < New_Image_Section->edgeDistance)
         New_Image_Section->edgeDistance = bottomEdgeDistance;

      rightEdgeDistance  =
            Header->section_x_length - imageSectionRightEdge;
      if (rightEdgeDistance < New_Image_Section->edgeDistance)
         New_Image_Section->edgeDistance = rightEdgeDistance;

      topEdgeDistance    =
            Header->section_y_length - imageSectionTopEdge;
      if (topEdgeDistance < New_Image_Section->edgeDistance)
         New_Image_Section->edgeDistance = topEdgeDistance;

      New_Image_Section->fullyContained =
            New_Image_Section->edgeDistance >= 0;
   }
   else
      return ( IMAGE_SECTION )NULL;

   return New_Image_Section;
}
/****************************************
* Get_Image_Section end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     
*    Delete_Image_Section                                                     
*                                                                             
* File:                                                                       
*    imgsectn.c                                                               
*                                                                             
* Description:                                                                
*    Free memory previously allocated for an image section.                   
*                                                                             
* Return Value:                                                               
*    none                                                                     
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       Section                                                               
*          IMAGE_SECTION, Reference                                           
*             The image section we're deleting from memory.                   
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 15-JUN-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
******************************************************************************/
/************************************
* Delete_Image_Section declarations *
************************************/
void          Delete_Image_Section( Section )
IMAGE_SECTION Section;
{
/*********************************************
* Delete_Image_Section begin executable code *
*********************************************/
   free( Section );

   return;
}
/*******************************************
* Delete_Image_Section end executable code *
*******************************************/
