/******************************************************************************
* C Source: Lookup Table                                                      
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    This set of functions performs various manipulations on a lookup table   
*    defined as follows:                                                      
*                                                                             
*       A set of order pairs of values, where each pair is called an entry    
*       and where the first value of each entry is called the independent     
*       value and the second value of each entry is called the dependent      
*       such that no two entries have the same independent value.  Any        
*       number of entries may have the same dependent value.
*                                                                             
*    A lookup table is implemented here as a linked list of entries where     
*    each entry is a structure consisting of a pair of character strings
*    for the independent and dependent values.
*                                                                             
* Module List:                                                                
*    Add_LookupTbl_Entry_to_Tbl                                               
*       Add an entry to an existing lookup table.                             
*    Cmp_LookupTbl_EntryIndepndVal
*    Create_LookupTbl                                                          
*       Create a new, empty lookup table.                                     
*    Del_LookupTbl
*       Delete an existing lookup table.
*    Del_LookupTbl_Entry_from_Tbl
*       Find in a lookup table an entry and delete it from the table.
*    Del_LookupTbl_Entry
*       Delete a lookup table entry structure.
*    Find_LookupTbl_Entry                                                      
*       Find an entry in a lookup table and return the pointer to the entry's 
*       data value.                                                           
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1997, Association of Universities for Research in Astronomy,  
* Inc.  All Rights Reserved.                                                  
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "looktab.h"

#include "lnkd_lst.h"

/******************************************************************************
* Module:                                                                     
*    Add_LookupTbl_Entry_to_Tbl                                                       
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Add an entry to an existing lookup table.  If there is already an entry  
*    an entry in the table with the input independent value or if memory      
*    cannot be allocated, then the call will fail and return 0.  Otherwise,    
*    1 is returned.                                                           
*                                                                             
* Return Value:                                                               
*    1 if successful, otherwise 0.                                            
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       table
*          LOOK_TAB, value                                                    
*             The look up table to which the entry is being added.            
*       independent_value, value                                          
*          char *, value                                                      
*             Independent value of the entry.                  
*       dependent_value, value                                            
*          char *, value                                                      
*             Dependent value of the entry.                    
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/******************************************
* Add_LookupTbl_Entry_to_Tbl declarations *
******************************************/
int        Add_LookupTbl_Entry_to_Tbl
                 (table, independent_value, dependent_value)
LOOK_TAB   table            ;
char     * independent_value;
char     * dependent_value  ;
{
   LOOK_TAB_ENTRY new_entry;

/***************************************************
* Add_LookupTbl_Entry_to_Tbl begin executable code *
***************************************************/
      if ((table             == (LOOK_TAB)NULL) ||
          (independent_value == (char *  )NULL) ||
          (dependent_value   == (char *  )NULL)   )
         return 0;

   if (Find_LookupTbl_Entry(table, independent_value) == (char *)NULL)
   {
      new_entry = (LOOK_TAB_ENTRY)malloc(sizeof(new_entry));

/* *** Add_LookupTbl_Entry_to_Tbl ***                                      2 */


         if (new_entry == (LOOK_TAB_ENTRY)NULL)
	    return 0;

      new_entry->independent_value = 
         (char *)malloc(strlen(independent_value) + 1);
         if (new_entry->independent_value == (char *)NULL)
	 {
	    free(new_entry);
	    return 0;
	 }
      strcpy(new_entry->independent_value, independent_value);

      new_entry->dependent_value = 
         (char *)malloc(strlen(dependent_value) + 1);
         if (new_entry->dependent_value == (char *)NULL)
	 {
	    free(new_entry->independent_value);
	    free(new_entry);
	    return 0;
	 }
      strcpy(new_entry->dependent_value, dependent_value);

      Insert_Linked_List_Data(table, new_entry);
   
      return 1;
   }
   return 0;
}
/*************************************************
* Add_LookupTbl_Entry_to_Tbl end executable code *
*************************************************/

/******************************************************************************
* Module:                                                                     
*    Cmp_LookupTbl_EntryIndepndVal
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Compares the independent value of a lookup table entry with an input
*    value.
*                                                                             
* Return Value:                                                               
*    A number greater than, equal to or less than zero is returned depending on
*    if the lookup table entry's independent value is greater than, equal to or
*    less than then input value.
*                                                                             
* Arguments;                                                                  
*    Input Arguments:                                                         
*       entry
*          LOOK_TAB_ENTRY, value
*             The entry whose independent value is to be compared.
*       input_value
*          char *, value                                                      
*             The value to compare.
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 17-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 19-FEB-97 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*********************************************
* Cmp_LookupTbl_EntryIndepndVal declarations *
*********************************************/
int              Cmp_LookupTbl_EntryIndepndVal(entry, input_value)
LOOK_TAB_ENTRY   entry      ;
char           * input_value;
{
   BOOL no_entry_independent_value;
   BOOL no_input_value;

/******************************************************
* Cmp_LookupTbl_EntryIndepndVal begin executable code *
******************************************************/
   if (entry == (LOOK_TAB_ENTRY)NULL)
      no_entry_independent_value = TRUE;

   else if (entry->independent_value == (char *)NULL)
      no_entry_independent_value = TRUE;

   else
      no_entry_independent_value = FALSE;


   if (input_value == (char *)NULL)
      no_input_value = TRUE;
   else

/* *** Cmp_LookupTbl_EntryIndepndVal ***                                   2 */


      no_input_value = FALSE;

   if (no_entry_independent_value && !no_input_value)
      return -1;

   else if ( no_entry_independent_value &&  no_input_value)
      return 0;

   else if (!no_entry_independent_value &&  no_input_value)
      return 1;

   else if (!no_entry_independent_value && !no_input_value)
   {
      if (entry->independent_value == input_value)
	 return (int)0;
      else
         return strcmp(entry->independent_value, input_value);
   }
}
/****************************************************
* Cmp_LookupTbl_EntryIndepndVal end executable code *
****************************************************/

/******************************************************************************
* Module:                                                                     
*    Create_LookupTbl                                                          
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Creates a new, empty lookup table.  If memory cannot be allocated, then
*    the call will fail and (LOOK_TAB)NULL will be returned.
*                                                                             
* Return Value:                                                               
*    If successful, the lookup table, otherwise (LOOK_TAB)NULL.
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       none
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*******************************
* Create_LookupTbl declarations *
*******************************/
LOOK_TAB Create_LookupTbl()
{
/****************************************
* Create_LookupTbl begin executable code *
****************************************/
   return (LOOK_TAB)Create_Empty_Linked_List();
}
/**************************************
* Create_LookupTbl end executable code *
**************************************/

/******************************************************************************
* Module:                                                                     
*    Del_LookupTbl
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Delete an existing lookup table.
*                                                                             
* Return Value:                                                               
*    none
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       table
*          LOOK_TAB, value                                                    
*             The look up table to which the entry is being added.            
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/*****************************
* Del_LookupTbl declarations *
*****************************/
void     Del_LookupTbl(table)
LOOK_TAB table;
{
/**************************************
* Del_LookupTbl begin executable code *
**************************************/
   Delete_Linked_List_Data(table, Del_LookupTbl_Entry);
}
/************************************
* Del_LookupTbl end executable code *
************************************/

/******************************************************************************
* Module:                                                                     
*    Del_LookupTbl_Entry
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Delete a lookup table entry.
*                                                                             
* Return Value:                                                               
*    none
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       entry
*          LOOK_TAB_ENTRY, value                                              
*             The entry to be deleted.
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/***********************************
* Del_LookupTbl_Entry declarations *
***********************************/
void           Del_LookupTbl_Entry(entry)
LOOK_TAB_ENTRY entry;
{
/********************************************
* Del_LookupTbl_Entry begin executable code *
********************************************/
   if (entry == (LOOK_TAB_ENTRY)NULL)
      return;

   if (entry->independent_value != (char *)NULL)
      free(entry->independent_value);

   if (entry->dependent_value != (char *)NULL)
      free(entry->dependent_value);

   free(entry);
}
/******************************************
* Del_LookupTbl_Entry end executable code *
******************************************/

/******************************************************************************
* Module:                                                                     
*    Del_LookupTbl_Entry_from_Tbl
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Find in a lookup table an entry and delete it.
*                                                                             
* Return Value:                                                               
*    none
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       table
*          LOOK_TAB, value                                                    
*             The look up table to which the entry is being added.            
*       independent_value
*          char *, value                                                      
*             Independent value of the entry.                  
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/********************************************
* Del_LookupTbl_Entry_from_Tbl declarations *
********************************************/
void       Del_LookupTbl_Entry_from_Tbl(table, independent_value)
LOOK_TAB   table            ;
char     * independent_value;
{
   LOOK_TAB_NODE  entry_node;
   LOOK_TAB_ENTRY entry     ;

/*****************************************************
* Del_LookupTbl_Entry_from_Tbl begin executable code *
*****************************************************/
   if (table == (LOOK_TAB)NULL)
      return;

   entry_node =
      (LOOK_TAB_NODE)Find_Linked_List_Node
         (table->Top                   ,
	  independent_value            ,
	  Cmp_LookupTbl_EntryIndepndVal);

   if (entry_node == (LOOK_TAB_NODE)NULL)
      return;

   entry = (LOOK_TAB_ENTRY)(entry_node->Data);
   Del_LookupTbl_Entry(entry);


/* *** Del_LookupTbl_Entry_from_Tbl ***                                    2 */


   Delete_Linked_List_Node(table, entry_node);

   return;
}
/***************************************************
* Del_LookupTbl_Entry_from_Tbl end executable code *
***************************************************/

/******************************************************************************
* Module:                                                                     
*    Find_LookupTbl_Entry
*                                                                             
* File:                                                                       
*    looktab.c                                                                
*                                                                             
* Description:                                                                
*    Find an entry in a lookup table and return the pointer to the entry's 
*    data value.  If the entry is not found, then (char *)NULL is returned.
*                                                                             
* Return Value:                                                               
*    Pointer to the entry's dependent value.  If the entry is not found, then
*    (char *)NULL is returned.
*                                                                             
* Arguments;                                                                  
*                                                                             
*    Input Arguments:                                                         
*       table
*          LOOK_TAB, value                                                    
*             The look up table in which the entry is being searched.
*       independent_value
*          char *, value
*             The independent value for which the lookup table is being
*             searched.
*    Input/Output Arguments:                                                  
*       none                                                                  
*    Output Arguments:                                                        
*       none                                                                  
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 12-SEP-96 by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
******************************************************************************/
/************************************
* Find_LookupTbl_Entry declarations *
************************************/
char     * Find_LookupTbl_Entry(table, independent_value)
LOOK_TAB   table            ;
char     * independent_value;
{
   LOOK_TAB_NODE  entry_node;
   LOOK_TAB_ENTRY entry     ;

/*********************************************
* Find_LookupTbl_Entry begin executable code *
*********************************************/
   entry_node = 
      Find_Linked_List_Node
         (table->Top                   ,
	  independent_value            ,
	  Cmp_LookupTbl_EntryIndepndVal);

   if (entry_node == (LOOK_TAB_NODE)NULL)
      return (char *)NULL;

   entry = (LOOK_TAB_ENTRY)(entry_node->Data);

   return entry->dependent_value;

/* *** Add_LookupTbl_Entry_to_Tbl ***                                      2 */


}
/*******************************************
* Find_LookupTbl_Entry end executable code *
*******************************************/
